/*
 * Name:    opt_differentiate.c
 *
 * Purpose: Compute the first derivative of the second column of a file
 *          with respect to the first column.
 *
 *--------------------------------------------------------------------
 *
 * Copyright 2001 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */


#include <stdio.h>
#include <math.h>

int
main( int argc, char *argv[] )
{
	const char pname[] = "opt_differentiate";

	FILE *input_file, *output_file;
	char buffer[80];
	double x_value, y_value;
	double old_x_value, old_y_value;
	double x_center_of_bin;
	double delta_x, delta_y;
	double deriv, old_deriv;
	int num_items, first_time;

	if ( argc != 3 ) {
		fprintf( stderr,
			"Usage: %s input_filename output_filename\n", pname );
		exit(1);
	}

	/* Open the input file. */

	input_file = fopen( argv[1], "r" );

	if ( input_file == NULL ) {
		fprintf( stderr,
			"%s: Cannot open input file '%s'\n",
			pname, argv[1] );
		exit(1);
	}

	/* Open the output file. */

	output_file = fopen( argv[2], "w" );

	if ( output_file == NULL ) {
		fprintf( stderr,
			"%s: Cannot open output file '%s'\n",
			pname, argv[2] );
		exit(1);
	}

	first_time = 1;
	x_value = 0.0;
	y_value = 0.0;
	old_x_value = 0.0;
	old_y_value = 0.0;
	deriv = 0.0;
	old_deriv = 0.0;

	fgets( buffer, sizeof buffer, input_file );

	while ( !feof(input_file) && !ferror(input_file) ) {

		num_items = sscanf( buffer, "%lg %lg", &x_value, &y_value );

		if ( num_items != 2 ) {
			fprintf( stderr,
			"%s: Cannot parse line '%s' in input_file '%s'\n",
			pname, buffer, argv[1] );
			exit(1);
		}

		if ( first_time ) {
			first_time = 0;
		} else {
			delta_y = y_value - old_y_value;
			delta_x = x_value - old_x_value;

			if ( fabs(delta_x) < 1.0e-308 ) {
				deriv = old_deriv;
			} else {
				deriv =  delta_y / delta_x;
			}

			x_center_of_bin = 0.5 * ( old_x_value + x_value );

			fprintf( output_file, "%g  %g\n",
				x_center_of_bin, deriv );
		}

		old_x_value = x_value;
		old_y_value = y_value;
		old_deriv = deriv;

		fgets( buffer, sizeof buffer, input_file );
	}

	fclose(input_file);
	fclose(output_file);

	exit(0);
}
